@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-document-saver.c,v 1.13 2006/09/28 09:47:01 pborelli Exp $
+ * $Id: gedit-document-saver.c 5320 2006-11-26 13:13:14Z sfre $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -191,7 +191,6 @@ write_document_contents (gint                  fd,
 	GtkTextIter end_iter;
 	gchar *contents;
 	gsize len;
-	gboolean add_cr;
 	ssize_t written;
 	gboolean res;
 
@@ -202,11 +201,6 @@ write_document_contents (gint                  fd,
 
 	len = strlen (contents);
 
-	if (len >= 1)
-		add_cr = (*(contents + len - 1) != '\n');
-	else
-		add_cr = FALSE;
-
 	if (encoding != gedit_encoding_get_utf8 ())
 	{
 		gchar *converted_contents;
@@ -263,8 +257,9 @@ write_document_contents (gint                  fd,
 		while (to_write > 0);
 	}
 
-	/* Add \n at the end if needed */
-	if (res && add_cr)
+	/* make sure files are always terminated with \n (see bug #95676). Note
+	that we strip the trailing \n when loading the file */
+	if (res)
 	{
 		if (encoding != gedit_encoding_get_utf8 ())
 		{
@@ -427,10 +422,12 @@ copy_file_data (gint     sfd,
 			bytes_written = write (dfd, write_buffer, bytes_to_write);
 			if (bytes_written == -1)
 			{
+				GnomeVFSResult result;
+
 				if (errno == EINTR)
 					continue;
 
-				GnomeVFSResult result = gnome_vfs_result_from_errno ();
+				result = gnome_vfs_result_from_errno ();
 
 				g_set_error (error,
 					     GEDIT_DOCUMENT_ERROR,
@@ -1289,11 +1286,11 @@ async_xfer_progress (GnomeVFSAsyncHandle      *handle,
 		     GnomeVFSXferProgressInfo *progress_info,
 		     gpointer                  data)
 {
+	GeditDocumentSaver *saver = GEDIT_DOCUMENT_SAVER (data);
+
 	gedit_debug_message (DEBUG_SAVER, "xfer phase: %d", progress_info->phase);
 	gedit_debug_message (DEBUG_SAVER, "xfer status: %d", progress_info->status);
 	
-	GeditDocumentSaver *saver = GEDIT_DOCUMENT_SAVER (data);
-
 	switch (progress_info->status)
 	{
 	case GNOME_VFS_XFER_PROGRESS_STATUS_OK:
