@@ -26,7 +26,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-prefs-manager-app.c,v 1.12 2006/01/19 18:16:00 paolo Exp $
+ * $Id: gedit-prefs-manager-app.c 5403 2007-01-02 17:10:40Z sfre $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -44,44 +44,49 @@
 #include "gedit-debug.h"
 #include "gedit-view.h"
 #include "gedit-window.h"
-#include "gedit-recent.h"
+#include "gedit-window-private.h"
 
 static void gedit_prefs_manager_editor_font_changed	(GConfClient *client,
-		   				 	 guint        cnxn_id,
-			   				 GConfEntry  *entry,
-			   				 gpointer     user_data);
+							 guint        cnxn_id,
+							 GConfEntry  *entry,
+							 gpointer     user_data);
 
-static void gedit_prefs_manager_editor_colors_changed 	(GConfClient *client,
-			   				 guint        cnxn_id,
-			   				 GConfEntry  *entry,
-			   				 gpointer     user_data);
+static void gedit_prefs_manager_system_font_changed	(GConfClient *client,
+							 guint        cnxn_id,
+							 GConfEntry  *entry,
+							 gpointer     user_data);
 
-static void gedit_prefs_manager_tabs_size_changed 	(GConfClient *client,
-						       	 guint        cnxn_id, 
-						       	 GConfEntry  *entry, 
-						       	 gpointer     user_data);
-				       	 
-static void gedit_prefs_manager_wrap_mode_changed 	(GConfClient *client,
+static void gedit_prefs_manager_editor_colors_changed	(GConfClient *client,
+							 guint        cnxn_id,
+							 GConfEntry  *entry,
+							 gpointer     user_data);
+
+static void gedit_prefs_manager_tabs_size_changed	(GConfClient *client,
 							 guint        cnxn_id, 
 							 GConfEntry  *entry, 
 							 gpointer     user_data);
 
-static void gedit_prefs_manager_line_numbers_changed 	(GConfClient *client,
+static void gedit_prefs_manager_wrap_mode_changed	(GConfClient *client,
 							 guint        cnxn_id, 
 							 GConfEntry  *entry, 
 							 gpointer     user_data);
 
-static void gedit_prefs_manager_auto_indent_changed 	(GConfClient *client,
+static void gedit_prefs_manager_line_numbers_changed	(GConfClient *client,
 							 guint        cnxn_id, 
 							 GConfEntry  *entry, 
 							 gpointer     user_data);
 
-static void gedit_prefs_manager_undo_changed 		(GConfClient *client,
+static void gedit_prefs_manager_auto_indent_changed	(GConfClient *client,
 							 guint        cnxn_id, 
 							 GConfEntry  *entry, 
 							 gpointer     user_data);
 
-static void gedit_prefs_manager_right_margin_changed 	(GConfClient *client,
+static void gedit_prefs_manager_undo_changed		(GConfClient *client,
+							 guint        cnxn_id, 
+							 GConfEntry  *entry, 
+							 gpointer     user_data);
+
+static void gedit_prefs_manager_right_margin_changed	(GConfClient *client,
 							 guint        cnxn_id, 
 							 GConfEntry  *entry, 
 							 gpointer     user_data);
@@ -107,7 +112,7 @@ static void gedit_prefs_manager_search_hl_enable_changed(GConfClient *client,
 							 gpointer     user_data);
 
 
-static void gedit_prefs_manager_max_recents_changed 	(GConfClient *client,
+static void gedit_prefs_manager_max_recents_changed	(GConfClient *client,
 							 guint        cnxn_id, 
 							 GConfEntry  *entry, 
 							 gpointer     user_data);
@@ -117,12 +122,21 @@ static void gedit_prefs_manager_auto_save_changed	(GConfClient *client,
 							 GConfEntry  *entry,
 							 gpointer     user_data);
 
+static void gedit_prefs_manager_lockdown_changed	(GConfClient *client,
+							 guint        cnxn_id,
+							 GConfEntry  *entry,
+							 gpointer     user_data);
+
+
 static gint window_state = -1;
 static gint window_height = -1;
 static gint window_width = -1;
 static gint side_panel_size = -1;
 static gint bottom_panel_size = -1;
 
+static gint side_panel_active_page = 0;
+static gint bottom_panel_active_page = 0;
+
 gboolean
 gedit_prefs_manager_app_init (void)
 {
@@ -140,11 +154,21 @@ gedit_prefs_manager_app_init (void)
 				GCONF_CLIENT_PRELOAD_RECURSIVE,
 				NULL);
 		
+		gconf_client_add_dir (gedit_prefs_manager->gconf_client,
+				GPM_LOCKDOWN_DIR,
+				GCONF_CLIENT_PRELOAD_RECURSIVE,
+				NULL);
+		
 		gconf_client_notify_add (gedit_prefs_manager->gconf_client,
 				GPM_FONT_DIR,
 				gedit_prefs_manager_editor_font_changed,
 				NULL, NULL, NULL);
 
+		gconf_client_notify_add (gedit_prefs_manager->gconf_client,
+				GPM_SYSTEM_FONT,
+				gedit_prefs_manager_system_font_changed,
+				NULL, NULL, NULL);
+
 		gconf_client_notify_add (gedit_prefs_manager->gconf_client,
 				GPM_COLORS_DIR,
 				gedit_prefs_manager_editor_colors_changed,
@@ -209,6 +233,11 @@ gedit_prefs_manager_app_init (void)
 				GPM_SAVE_DIR,
 				gedit_prefs_manager_auto_save_changed,
 				NULL, NULL, NULL);
+		
+		gconf_client_notify_add (gedit_prefs_manager->gconf_client,
+				GPM_LOCKDOWN_DIR,
+				gedit_prefs_manager_lockdown_changed,
+				NULL, NULL, NULL);
 	}
 
 	return gedit_prefs_manager != NULL;	
@@ -250,69 +279,50 @@ gedit_prefs_manager_window_state_can_set (void)
 	return TRUE;
 }
 
-/* Window height */
-gint
-gedit_prefs_manager_get_window_height (void)
-{
-	if (window_height == -1)
-		window_height = gnome_config_get_int (GPM_WINDOW_HEIGHT "=" GPM_DEFAULT_WINDOW_HEIGHT_STR);
-
-	return window_height;
-}
-
-gint
-gedit_prefs_manager_get_default_window_height (void)
-{
-	return GPM_DEFAULT_WINDOW_HEIGHT;
-}
-
+/* Window size */
 void
-gedit_prefs_manager_set_window_height (gint wh)
+gedit_prefs_manager_get_window_size (gint *width, gint *height)
 {
-	g_return_if_fail (wh > -1);
+	g_return_if_fail (width != NULL && height != NULL);
 
-	window_height = wh;
-	gnome_config_set_int (GPM_WINDOW_HEIGHT, wh);
-}
-
-gboolean 
-gedit_prefs_manager_window_height_can_set (void)
-{
-	return TRUE;
-}
-
-/* Window width */
-gint
-gedit_prefs_manager_get_window_width (void)
-{
 	if (window_width == -1)
 		window_width = gnome_config_get_int (GPM_WINDOW_WIDTH "=" GPM_DEFAULT_WINDOW_WIDTH_STR);
 
-	return window_width;
+	if (window_height == -1)
+		window_height = gnome_config_get_int (GPM_WINDOW_HEIGHT "=" GPM_DEFAULT_WINDOW_HEIGHT_STR);
+
+	*width = window_width;
+	*height = window_height;
 }
 
-gint 
-gedit_prefs_manager_get_default_window_width (void)
+void
+gedit_prefs_manager_get_default_window_size (gint *width, gint *height)
 {
-	return GPM_DEFAULT_WINDOW_WIDTH;
+	g_return_if_fail (width != NULL && height != NULL);
+
+	*width = GPM_DEFAULT_WINDOW_WIDTH;
+	*height = GPM_DEFAULT_WINDOW_HEIGHT;
 }
 
-void 
-gedit_prefs_manager_set_window_width (gint ww)
+void
+gedit_prefs_manager_set_window_size (gint width, gint height)
 {
-	g_return_if_fail (ww > -1);
-	
-	window_width = ww;
-	gnome_config_set_int (GPM_WINDOW_WIDTH, ww);
+	g_return_if_fail (width > -1 && height > -1);
+
+	window_width = width;
+	window_height = height;
+
+	gnome_config_set_int (GPM_WINDOW_WIDTH, width);
+	gnome_config_set_int (GPM_WINDOW_HEIGHT, height);
 }
 
 gboolean 
-gedit_prefs_manager_window_width_can_set (void)
+gedit_prefs_manager_window_size_can_set (void)
 {
 	return TRUE;
 }
 
-/* Side panel size */
+/* Side panel */
 gint
 gedit_prefs_manager_get_side_panel_size (void)
 {
@@ -346,7 +356,33 @@ gedit_prefs_manager_side_panel_size_can_set (void)
 	return TRUE;
 }
 
-/* Bottom panel size */
+gint
+gedit_prefs_manager_get_side_panel_active_page (void)
+{
+	if (side_panel_active_page == 0)
+		side_panel_active_page = gnome_config_get_int (
+				GPM_SIDE_PANEL_ACTIVE_PAGE);
+
+	return side_panel_active_page;
+}
+
+void
+gedit_prefs_manager_set_side_panel_active_page (gint id)
+{
+	if (side_panel_active_page == id)
+		return;
+
+	side_panel_active_page = id;
+	gnome_config_set_int (GPM_SIDE_PANEL_ACTIVE_PAGE, id);
+}
+
+gboolean 
+gedit_prefs_manager_side_panel_active_page_can_set (void)
+{
+	return TRUE;
+}
+
+/* Bottom panel */
 gint
 gedit_prefs_manager_get_bottom_panel_size (void)
 {
@@ -380,6 +416,32 @@ gedit_prefs_manager_bottom_panel_size_can_set (void)
 	return TRUE;
 }
 
+gint
+gedit_prefs_manager_get_bottom_panel_active_page (void)
+{
+	if (bottom_panel_active_page == 0)
+		bottom_panel_active_page = gnome_config_get_int (
+				GPM_BOTTOM_PANEL_ACTIVE_PAGE);
+
+	return bottom_panel_active_page;
+}
+
+void
+gedit_prefs_manager_set_bottom_panel_active_page (gint id)
+{
+	if (bottom_panel_active_page == id)
+		return;
+
+	bottom_panel_active_page = id;
+	gnome_config_set_int (GPM_BOTTOM_PANEL_ACTIVE_PAGE, id);
+}
+
+gboolean 
+gedit_prefs_manager_bottom_panel_active_page_can_set (void)
+{
+	return TRUE;
+}
+
 static void 
 gedit_prefs_manager_editor_font_changed (GConfClient *client,
 					 guint        cnxn_id, 
@@ -421,6 +483,8 @@ gedit_prefs_manager_editor_font_changed (GConfClient *client,
 
 	if ((font == NULL) && !def)
 		font = gedit_prefs_manager_get_editor_font ();
+	else
+		font = gedit_prefs_manager_get_system_font ();
 
 	ts = gedit_prefs_manager_get_tabs_size ();
 
@@ -429,11 +493,9 @@ gedit_prefs_manager_editor_font_changed (GConfClient *client,
 
 	while (l != NULL)
 	{
-		gedit_view_set_font (GEDIT_VIEW (l->data),
-				     def, 
-				     font);
-		gtk_source_view_set_tabs_width (GTK_SOURCE_VIEW (l->data),
-						ts);
+		/* Note: we use def=FALSE to avoid GeditView to query gconf */
+		gedit_view_set_font (GEDIT_VIEW (l->data), FALSE,  font);
+		gtk_source_view_set_tabs_width (GTK_SOURCE_VIEW (l->data), ts);
 
 		l = l->next;
 	}
@@ -442,6 +504,51 @@ gedit_prefs_manager_editor_font_changed (GConfClient *client,
 	g_free (font);
 }
 
+static void 
+gedit_prefs_manager_system_font_changed (GConfClient *client,
+					 guint        cnxn_id, 
+					 GConfEntry  *entry, 
+					 gpointer     user_data)
+{
+	GList *views;
+	GList *l;
+	gchar *font;
+	gint ts;
+
+	gedit_debug (DEBUG_PREFS);
+
+	g_return_if_fail (entry->key != NULL);
+	g_return_if_fail (entry->value != NULL);
+
+	if (strcmp (entry->key, GPM_SYSTEM_FONT) != 0)
+		return;
+
+	if (!gedit_prefs_manager_get_use_default_font ())
+		return;
+
+	if (entry->value->type == GCONF_VALUE_STRING)
+		font = g_strdup (gconf_value_get_string (entry->value));
+	else
+		font = g_strdup (GPM_DEFAULT_SYSTEM_FONT);
+
+	ts = gedit_prefs_manager_get_tabs_size ();
+
+	views = gedit_app_get_views (gedit_app_get_default ());
+	l = views;
+
+	while (l != NULL)
+	{
+		/* Note: we use def=FALSE to avoid GeditView to query gconf */
+		gedit_view_set_font (GEDIT_VIEW (l->data), FALSE, font);
+
+		gtk_source_view_set_tabs_width (GTK_SOURCE_VIEW (l->data), ts);
+		l = l->next;
+	}
+
+	g_list_free (views);
+	g_free (font);
+}
+
 static void 
 set_colors (gboolean  def, 
 	    GdkColor *backgroud, 
@@ -1040,7 +1147,7 @@ gedit_prefs_manager_search_hl_enable_changed (GConfClient *client,
 			g_return_if_fail (GEDIT_IS_DOCUMENT (l->data));
 
 			gedit_document_set_enable_search_highlighting  (GEDIT_DOCUMENT (l->data),
-							 		enable);
+									enable);
 
 			l = l->next;
 		}
@@ -1062,7 +1169,7 @@ gedit_prefs_manager_max_recents_changed (GConfClient *client,
 
 	if (strcmp (entry->key, GPM_MAX_RECENTS) == 0)
 	{
-		EggRecentModel *model;
+		const GList *windows;
 		gint max;
 
 		if (entry->value->type == GCONF_VALUE_INT)
@@ -1075,9 +1182,19 @@ gedit_prefs_manager_max_recents_changed (GConfClient *client,
 		else
 			max = GPM_DEFAULT_MAX_RECENTS;
 
-		model = gedit_recent_get_model ();
+		windows = gedit_app_get_windows (gedit_app_get_default ());
+		while (windows != NULL)
+		{
+			GeditWindow *w = windows->data;
 
-		egg_recent_model_set_limit (model, max);
+			gtk_recent_chooser_set_limit (GTK_RECENT_CHOOSER (w->priv->toolbar_recent_menu),
+						      max);
+
+			windows = g_list_next (windows);
+		}
+
+		/* FIXME: we have no way at the moment to trigger the
+		 * update of the inline recents in the File menu */
 	}
 }
 
@@ -1149,3 +1266,45 @@ gedit_prefs_manager_auto_save_changed (GConfClient *client,
 		g_list_free (docs);
 	}
 }
+
+static void
+gedit_prefs_manager_lockdown_changed (GConfClient *client,
+				      guint        cnxn_id,
+				      GConfEntry  *entry,
+				      gpointer     user_data)
+{
+	GeditApp *app;
+	gboolean locked;
+
+	gedit_debug (DEBUG_PREFS);
+
+	g_return_if_fail (entry->key != NULL);
+	g_return_if_fail (entry->value != NULL);
+
+	if (entry->value->type == GCONF_VALUE_BOOL)
+		locked = gconf_value_get_bool (entry->value);
+	else
+		locked = FALSE;
+
+	app = gedit_app_get_default ();
+
+	if (strcmp (entry->key, GPM_LOCKDOWN_COMMAND_LINE) == 0)
+		_gedit_app_set_lockdown_bit (app, 
+					     GEDIT_LOCKDOWN_COMMAND_LINE,
+					     locked);
+
+	else if (strcmp (entry->key, GPM_LOCKDOWN_PRINTING) == 0)
+		_gedit_app_set_lockdown_bit (app, 
+					     GEDIT_LOCKDOWN_PRINTING,
+					     locked);
+
+	else if (strcmp (entry->key, GPM_LOCKDOWN_PRINT_SETUP) == 0)
+		_gedit_app_set_lockdown_bit (app, 
+					     GEDIT_LOCKDOWN_PRINT_SETUP,
+					     locked);
+
+	else if (strcmp (entry->key, GPM_LOCKDOWN_SAVE_TO_DISK) == 0)
+		_gedit_app_set_lockdown_bit (app, 
+					     GEDIT_LOCKDOWN_SAVE_TO_DISK,
+					     locked);
+}
