@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-prefs-manager-app.h,v 1.2 2005/12/12 17:27:55 pborelli Exp $
+ * $Id: gedit-prefs-manager-app.h 5244 2006-09-23 14:37:09Z pborelli $
  *
  */
 
@@ -48,28 +48,32 @@ gint		 gedit_prefs_manager_get_window_state		(void);
 void 		 gedit_prefs_manager_set_window_state		(gint ws);
 gboolean	 gedit_prefs_manager_window_state_can_set	(void);
 
-/* Window height */
-gint		 gedit_prefs_manager_get_window_height		(void);
-gint		 gedit_prefs_manager_get_default_window_height	(void);
-void 		 gedit_prefs_manager_set_window_height		(gint wh);
-gboolean	 gedit_prefs_manager_window_height_can_set	(void);
+/* Window size */
+void		 gedit_prefs_manager_get_window_size		(gint *width,
+								 gint *height);
+void		 gedit_prefs_manager_get_default_window_size	(gint *width,
+								 gint *height);
+void 		 gedit_prefs_manager_set_window_size		(gint width,
+								 gint height);
+gboolean	 gedit_prefs_manager_window_size_can_set	(void);
 
-/* Window width */
-gint	 	 gedit_prefs_manager_get_window_width		(void);
-gint	 	 gedit_prefs_manager_get_default_window_width	(void);
-void 		 gedit_prefs_manager_set_window_width		(gint ww);
-gboolean	 gedit_prefs_manager_window_width_can_set	(void);
-
-/* Side panel size */
+/* Side panel */
 gint	 	 gedit_prefs_manager_get_side_panel_size	(void);
 gint	 	 gedit_prefs_manager_get_default_side_panel_size(void);
 void 		 gedit_prefs_manager_set_side_panel_size	(gint ps);
 gboolean	 gedit_prefs_manager_side_panel_size_can_set	(void);
+gint		 gedit_prefs_manager_get_side_panel_active_page (void);
+void 		 gedit_prefs_manager_set_side_panel_active_page (gint id);
+gboolean	 gedit_prefs_manager_side_panel_active_page_can_set (void);
 
-/* Bottom panel size */
+/* Bottom panel */
 gint	 	 gedit_prefs_manager_get_bottom_panel_size	(void);
 gint	 	 gedit_prefs_manager_get_default_bottom_panel_size(void);
 void 		 gedit_prefs_manager_set_bottom_panel_size	(gint ps);
 gboolean	 gedit_prefs_manager_bottom_panel_size_can_set	(void);
+gint		 gedit_prefs_manager_get_bottom_panel_active_page (void);
+void 		 gedit_prefs_manager_set_bottom_panel_active_page (gint id);
+gboolean	 gedit_prefs_manager_bottom_panel_active_page_can_set (void);
+
 
 #endif /* __GEDIT_PREFS_MANAGER_APP_H__ */
