@@ -25,6 +25,7 @@
 #endif
 
 #include <gedit/gedit-commands.h>
+#include <gedit/gedit-utils.h>
 #include <glib/gi18n-lib.h>
 #include <gedit/gedit-debug.h>
 #include <gconf/gconf-client.h>
@@ -36,8 +37,12 @@
 #include "gedit-file-browser-error.h"
 #include "gedit-file-browser-widget.h"
 
-#define WINDOW_DATA_KEY	        "GeditFileBrowserPluginWindowData"
-#define FILE_BROWSER_BASE_KEY 	"/apps/gedit-2/plugins/filebrowser"
+#define WINDOW_DATA_KEY	        	"GeditFileBrowserPluginWindowData"
+#define FILE_BROWSER_BASE_KEY 		"/apps/gedit-2/plugins/filebrowser"
+#define NAUTILUS_CLICK_POLICY_BASE_KEY 	"/apps/nautilus/preferences"
+#define NAUTILUS_CLICK_POLICY_KEY	"click_policy"
+#define NAUTILUS_ENABLE_DELETE_KEY	"enable_delete"
+#define NAUTILUS_CONFIRM_TRASH_KEY	"confirm_trash"
 
 #define GEDIT_FILE_BROWSER_PLUGIN_GET_PRIVATE(object)	(G_TYPE_INSTANCE_GET_PRIVATE ((object), GEDIT_TYPE_FILE_BROWSER_PLUGIN, GeditFileBrowserPluginPrivate))
 
@@ -50,7 +55,15 @@ typedef struct _GeditFileBrowserPluginData
 {
 	GeditFileBrowserWidget * tree_widget;
 	gulong                   merge_id;
-	GtkActionGroup         * action_group;	
+	GtkActionGroup         * action_group;
+	GtkActionGroup	       * single_selection_action_group;
+	gboolean	         auto_root;
+	gulong                   end_loading_handle;
+	gboolean		 confirm_trash;
+
+	guint			 click_policy_handle;
+	guint			 enable_delete_handle;
+	guint			 confirm_trash_handle;
 } GeditFileBrowserPluginData;
 
 static void on_uri_activated_cb          (GeditFileBrowserWidget * widget,
@@ -77,12 +90,15 @@ static void on_tab_added_cb              (GeditWindow * window,
                                           GeditFileBrowserPluginData * data);
 static gboolean on_confirm_delete_cb     (GeditFileBrowserWidget * widget,
                                           GeditFileBrowserStore * store,
-                                          GtkTreeIter * iter,
+                                          GList * rows,
                                           GeditWindow * window);
 static gboolean on_confirm_no_trash_cb   (GeditFileBrowserWidget * widget,
                                           GeditFileBrowserStore * store,
-                                          GtkTreeIter * iter,
+                                          GList * rows,
                                           GeditWindow * window);
+static void on_end_loading_cb            (GeditFileBrowserStore      * store,
+                                          GtkTreeIter                * iter,
+                                          GeditFileBrowserPluginData * data);
 
 GEDIT_PLUGIN_REGISTER_TYPE_WITH_CODE (GeditFileBrowserPlugin, filetree_plugin, 	\
 	gedit_file_browser_enum_and_flag_register_type (module);		\
@@ -113,12 +129,32 @@ get_plugin_data (GeditWindow * window)
 	return (GeditFileBrowserPluginData *) (g_object_get_data (G_OBJECT (window), WINDOW_DATA_KEY));
 }
 
+static void
+prepare_auto_root (GeditFileBrowserPluginData *data)
+{
+	GeditFileBrowserStore *store;
+
+	data->auto_root = TRUE;
+	
+	store = gedit_file_browser_widget_get_browser_store (data->tree_widget);
+	
+	if (data->end_loading_handle != 0) {
+		g_signal_handler_disconnect(store, data->end_loading_handle);
+		data->end_loading_handle = 0;
+	}
+
+	data->end_loading_handle = g_signal_connect (store, 
+	                                             "end-loading",
+	                                             G_CALLBACK (on_end_loading_cb),
+	                                             data);
+}
+
 static void 
 restore_default_location (GeditFileBrowserPlugin * plugin, 
                           GeditFileBrowserPluginData *data)
 {
 	gchar * root;
-	gchar * virtual;
+	gchar * virtual_root;
 	gboolean bookmarks;
 	gboolean remote;
 	GConfClient * client;
@@ -142,7 +178,7 @@ restore_default_location (GeditFileBrowserPlugin * plugin,
 	root = gconf_client_get_string (client, 
 	                                FILE_BROWSER_BASE_KEY "/on_load/root", 
 	                                NULL);
-	virtual = gconf_client_get_string (client, 
+	virtual_root = gconf_client_get_string (client, 
 	                                   FILE_BROWSER_BASE_KEY "/on_load/virtual_root", 
 	                                   NULL);
 
@@ -153,12 +189,14 @@ restore_default_location (GeditFileBrowserPlugin * plugin,
 	if (root != NULL && *root != '\0') {
 		uri = gnome_vfs_uri_new (root);
 
-		if (uri == NULL || (!remote && !gedit_file_browser_utils_is_local (root))) {
-		} else if (virtual != NULL && virtual != '\0') {
+		if (uri == NULL || (!remote && !gedit_utils_uri_has_file_scheme (root))) {
+		} else if (virtual_root != NULL && virtual_root != '\0') {
+			prepare_auto_root(data);
 			gedit_file_browser_widget_set_root_and_virtual_root (data->tree_widget, 
 			                                                     root,
-			                                                     virtual);
+			                                                     virtual_root);
 		} else {
+			prepare_auto_root(data);
 			gedit_file_browser_widget_set_root (data->tree_widget,
 			                                    root,
 			                                    TRUE);
@@ -170,7 +208,7 @@ restore_default_location (GeditFileBrowserPlugin * plugin,
 
 	g_object_unref (client);
 	g_free (root);
-	g_free (virtual);
+	g_free (virtual_root);
 }
 
 static void
@@ -226,31 +264,174 @@ restore_filter (GeditFileBrowserPluginData * data)
 	g_free (pattern);
 }
 
+static GeditFileBrowserViewClickPolicy
+click_policy_from_string (gchar const *click_policy)
+{
+	if (click_policy && strcmp (click_policy, "single") == 0)
+		return GEDIT_FILE_BROWSER_VIEW_CLICK_POLICY_SINGLE;
+	else
+		return GEDIT_FILE_BROWSER_VIEW_CLICK_POLICY_DOUBLE;
+}
+
+static void
+on_click_policy_changed (GConfClient *client,
+			 guint cnxn_id,
+			 GConfEntry *entry,
+			 gpointer user_data)
+{
+	GConfValue *value;
+	GeditFileBrowserPluginData * data;
+	gchar const *click_policy;
+	GeditFileBrowserViewClickPolicy policy = GEDIT_FILE_BROWSER_VIEW_CLICK_POLICY_DOUBLE;
+	GeditFileBrowserView *view;
+
+	data = (GeditFileBrowserPluginData *)(user_data);	
+	value = gconf_entry_get_value (entry);
+	
+	if (value && value->type == GCONF_VALUE_STRING) {
+		click_policy = gconf_value_get_string (value);
+		
+		policy = click_policy_from_string (click_policy);
+	}
+	
+	view = gedit_file_browser_widget_get_browser_view (data->tree_widget);
+	gedit_file_browser_view_set_click_policy (view, policy);	
+}
+
+static void
+on_enable_delete_changed (GConfClient *client,
+		 	  guint cnxn_id,
+			  GConfEntry *entry,
+			  gpointer user_data)
+{
+	GConfValue *value;
+	GeditFileBrowserPluginData *data;
+	gboolean enable = FALSE;
+
+	data = (GeditFileBrowserPluginData *)(user_data);	
+	value = gconf_entry_get_value (entry);
+	
+	if (value && value->type == GCONF_VALUE_BOOL)
+		enable = gconf_value_get_bool (value);
+
+	g_object_set (G_OBJECT (data->tree_widget), "enable-delete", enable, NULL);
+}
+
+static void
+on_confirm_trash_changed (GConfClient *client,
+		 	  guint cnxn_id,
+			  GConfEntry *entry,
+			  gpointer user_data)
+{
+	GConfValue *value;
+	GeditFileBrowserPluginData *data;
+	gboolean enable = FALSE;
+
+	data = (GeditFileBrowserPluginData *)(user_data);	
+	value = gconf_entry_get_value (entry);
+	
+	if (value && value->type == GCONF_VALUE_BOOL)
+		enable = gconf_value_get_bool (value);
+
+	data->confirm_trash = enable;
+}
+
+static void
+install_nautilus_prefs (GeditFileBrowserPluginData *data)
+{
+	GConfClient * client;
+	gchar *pref;
+	gboolean prefb;
+	client = gconf_client_get_default ();
+	GeditFileBrowserViewClickPolicy policy;
+	GeditFileBrowserView *view;
+
+	if (!client)
+		return;
+
+	gconf_client_add_dir (client, 
+			      NAUTILUS_CLICK_POLICY_BASE_KEY,
+			      GCONF_CLIENT_PRELOAD_NONE,
+			      NULL);
+
+	/* Get click_policy */
+	pref = gconf_client_get_string (client,
+	                                NAUTILUS_CLICK_POLICY_BASE_KEY "/" NAUTILUS_CLICK_POLICY_KEY,
+	                                NULL);
+	
+	policy = click_policy_from_string (pref);
+	
+	view = gedit_file_browser_widget_get_browser_view (data->tree_widget);
+	gedit_file_browser_view_set_click_policy (view, policy);
+	
+	if (pref) {
+		data->click_policy_handle = 
+			gconf_client_notify_add (client, 
+						 NAUTILUS_CLICK_POLICY_BASE_KEY "/" NAUTILUS_CLICK_POLICY_KEY,
+						 on_click_policy_changed,
+						 data,
+						 NULL,
+						 NULL);
+		g_free (pref);
+	}
+	
+	/* Get enable_delete */
+	prefb = gconf_client_get_bool (client,
+	                               NAUTILUS_CLICK_POLICY_BASE_KEY "/" NAUTILUS_ENABLE_DELETE_KEY,
+	                               NULL);
+	
+	g_object_set (G_OBJECT (data->tree_widget), "enable-delete", prefb, NULL);
+
+	data->enable_delete_handle = 
+			gconf_client_notify_add (client, 
+						 NAUTILUS_CLICK_POLICY_BASE_KEY "/" NAUTILUS_ENABLE_DELETE_KEY,
+						 on_enable_delete_changed,
+						 data,
+						 NULL,
+						 NULL);
+
+	/* Get confirm_trash */
+	prefb = gconf_client_get_bool (client,
+	                               NAUTILUS_CLICK_POLICY_BASE_KEY "/" NAUTILUS_CONFIRM_TRASH_KEY,
+	                               NULL);
+	
+	data->confirm_trash = prefb;
+
+	data->confirm_trash_handle = 
+			gconf_client_notify_add (client, 
+						 NAUTILUS_CLICK_POLICY_BASE_KEY "/" NAUTILUS_CONFIRM_TRASH_KEY,
+						 on_confirm_trash_changed,
+						 data,
+						 NULL,
+						 NULL);
+	g_object_unref (client);
+}
+
 static void
 set_root_from_doc (GeditFileBrowserPluginData * data,
                    GeditDocument * doc)
 {
-	gchar * uri;
-	gchar * root;
-	GnomeVFSURI * guri;
-	GnomeVFSURI * parent;
+	gchar *uri;
+	gchar *root;
+	GnomeVFSURI *guri;
 
 	if (doc == NULL)
 		return;
-	
-	uri = gedit_document_get_uri (doc);
 
+	uri = gedit_document_get_uri (doc);
 	if (uri == NULL)
 		return;
-	
+
 	guri = gnome_vfs_uri_new (uri);
-	
+
 	if (guri == NULL) {
 		g_free (uri);
 		return;
 	}
-	
-	if (gnome_vfs_uri_has_parent(guri)) {
+
+	if (gnome_vfs_uri_has_parent (guri)) {
+		GnomeVFSURI *parent;
+
 		parent = gnome_vfs_uri_get_parent (guri);
 		gnome_vfs_uri_unref (guri);
 		guri = parent;
@@ -261,6 +442,8 @@ set_root_from_doc (GeditFileBrowserPluginData * data,
 	gedit_file_browser_widget_set_root (data->tree_widget,
 		                            root,
 		                            TRUE);
+
+	gnome_vfs_uri_unref (guri);
 	g_free (root);
 	g_free (uri);
 }
@@ -269,15 +452,15 @@ static void
 on_action_set_active_root (GtkAction * action,
                            GeditWindow * window)
 {
-	GeditFileBrowserPluginData * data;
-	
+	GeditFileBrowserPluginData *data;
+
 	data = get_plugin_data (window);
 	set_root_from_doc (data, 
 	                   gedit_window_get_active_document (window));
 }
 
 static gchar *
-get_terminal ()
+get_terminal (void)
 {
 	GConfClient * client;
 	gchar * terminal;
@@ -343,6 +526,54 @@ on_action_open_terminal (GtkAction * action,
 	g_free (local);
 }
 
+static void
+on_selection_changed_cb (GtkTreeSelection *selection,
+			 GeditWindow      *window)
+{
+	GeditFileBrowserPluginData *data;
+	GtkTreeView *tree_view;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	gboolean sensitive;
+	GnomeVFSURI *guri;
+	gchar *uri;
+
+	data = get_plugin_data (window);
+	
+	tree_view = GTK_TREE_VIEW (gedit_file_browser_widget_get_browser_view (data->tree_widget));
+	model = gtk_tree_view_get_model (tree_view);
+	
+	if (!GEDIT_IS_FILE_BROWSER_STORE (model))
+		return;
+	
+	if (gedit_file_browser_widget_get_num_selected_files_or_directories (data->tree_widget) > 1)
+		sensitive = FALSE;
+	else if (!gedit_file_browser_widget_get_selected_directory (data->tree_widget, &iter))
+		sensitive = FALSE;
+	else
+		sensitive = TRUE;
+	
+	if (sensitive) {
+		gtk_tree_model_get (model, &iter, 
+				    GEDIT_FILE_BROWSER_STORE_COLUMN_URI, 
+				    &uri, -1);
+
+		guri = gnome_vfs_uri_new (uri);
+
+		sensitive = guri != NULL && gedit_utils_uri_has_file_scheme (uri);
+	
+		if (guri)
+			gnome_vfs_uri_unref (guri);
+
+		g_free (uri);
+	}
+
+	gtk_action_set_sensitive (
+		gtk_action_group_get_action (data->single_selection_action_group, 
+                                            "OpenTerminal"),
+		sensitive);
+}
+
 #define POPUP_UI ""                          \
 "<ui>"                                       \
 "  <popup name=\"Popup\">"                   \
@@ -360,11 +591,14 @@ static GtkActionEntry extra_actions[] =
 	{"SetActiveRoot", GTK_STOCK_JUMP_TO, N_("_Set root to active document"),
 	 NULL,
 	 N_("Set the root to the active document location"),
-	 G_CALLBACK (on_action_set_active_root)},
+	 G_CALLBACK (on_action_set_active_root)}
+};
+
+static GtkActionEntry extra_single_selection_actions[] = {
 	{"OpenTerminal", GTK_STOCK_EXECUTE, N_("_Open terminal here"),
 	 NULL,
 	 N_("Open a terminal at the currently opened directory"),
-	 G_CALLBACK (on_action_open_terminal)},
+	 G_CALLBACK (on_action_open_terminal)}
 };
 
 static void
@@ -385,6 +619,14 @@ add_popup_ui (GeditWindow * window)
 				      window);
 	gtk_ui_manager_insert_action_group (manager, action_group, 0);
 	data->action_group = action_group;
+	
+	action_group =
+	    gtk_action_group_new ("FileBrowserPluginSingleSelectionExtra");
+	gtk_action_group_add_actions (action_group, extra_single_selection_actions,
+				      G_N_ELEMENTS (extra_single_selection_actions),
+				      window);
+	gtk_ui_manager_insert_action_group (manager, action_group, 0);
+	data->single_selection_action_group = action_group;
 
 	data->merge_id = gtk_ui_manager_add_ui_from_string (manager, 
 	                                                    POPUP_UI, 
@@ -409,6 +651,9 @@ remove_popup_ui (GeditWindow * window)
 	
 	gtk_ui_manager_remove_action_group (manager, data->action_group);
 	g_object_unref (data->action_group);
+
+	gtk_ui_manager_remove_action_group (manager, data->single_selection_action_group);
+	g_object_unref (data->single_selection_action_group);
 }
 
 static void
@@ -437,6 +682,8 @@ impl_activate (GeditPlugin * plugin, GeditWindow * window)
 	GeditFileBrowserStore * store;
 
 	data = g_new (GeditFileBrowserPluginData, 1);
+	data->auto_root = FALSE;
+	data->end_loading_handle = 0;
 	data->tree_widget = GEDIT_FILE_BROWSER_WIDGET (gedit_file_browser_widget_new ());
 
 	g_signal_connect (data->tree_widget,
@@ -461,6 +708,13 @@ impl_activate (GeditPlugin * plugin, GeditWindow * window)
 	                  G_CALLBACK (on_confirm_no_trash_cb),
 	                  window);
 
+	g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW 
+			  (gedit_file_browser_widget_get_browser_view 
+			  (data->tree_widget))),
+			  "changed",
+			  G_CALLBACK (on_selection_changed_cb),
+			  window);			  
+
 	panel = gedit_window_get_side_panel (window);
 	pixbuf = gedit_file_browser_utils_pixbuf_from_theme("file-manager",  
 	                                                    GTK_ICON_SIZE_MENU);
@@ -488,6 +742,9 @@ impl_activate (GeditPlugin * plugin, GeditWindow * window)
 	/* Restore filter options */
 	restore_filter (data);
 	
+	/* Install nautilus preferences */
+	install_nautilus_prefs (data);
+	
 	/* Connect signals to store the last visited location */
 	g_signal_connect (gedit_file_browser_widget_get_browser_view (data->tree_widget),
 	                  "notify::model",
@@ -518,6 +775,7 @@ impl_deactivate (GeditPlugin * plugin, GeditWindow * window)
 {
 	GeditFileBrowserPluginData * data;
 	GeditPanel * panel;
+	GConfClient *client;
 
 	data = get_plugin_data (window);
 
@@ -526,6 +784,19 @@ impl_deactivate (GeditPlugin * plugin, GeditWindow * window)
 	                                      G_CALLBACK (on_tab_added_cb),
 	                                      data);
 
+	client = gconf_client_get_default ();
+	gconf_client_remove_dir (client, NAUTILUS_CLICK_POLICY_BASE_KEY, NULL);
+	
+	if (data->click_policy_handle)
+		gconf_client_notify_remove (client, data->click_policy_handle);
+
+	if (data->enable_delete_handle)
+		gconf_client_notify_remove (client, data->enable_delete_handle);
+
+	if (data->confirm_trash_handle)
+		gconf_client_notify_remove (client, data->confirm_trash_handle);
+
+	g_object_unref (client);
 	remove_popup_ui (window);
 
 	panel = gedit_window_get_side_panel (window);
@@ -565,6 +836,18 @@ on_error_cb (GeditFileBrowserWidget * tree_widget,
 {
 	gchar * title;
 	GtkWidget * dlg;
+	GeditFileBrowserPluginData * data;
+	
+	data = get_plugin_data (window);
+	
+	/* Do not show the error when the root has been set automatically */
+	if (data->auto_root && (code == GEDIT_FILE_BROWSER_ERROR_SET_ROOT ||
+	                        code == GEDIT_FILE_BROWSER_ERROR_LOAD_DIRECTORY)) 
+	{
+		/* Show bookmarks */
+		gedit_file_browser_widget_show_bookmarks (data->tree_widget);
+		return;
+	}
 
 	switch (code) {
 	case GEDIT_FILE_BROWSER_ERROR_NEW_DIRECTORY:
@@ -714,9 +997,8 @@ on_virtual_root_changed_cb (GeditFileBrowserStore * store,
 {
 	GeditFileBrowserPluginData * data = get_plugin_data (window);
 	gchar * root;
-	gchar * virtual;
+	gchar * virtual_root;
 	GConfClient * client;
-	GnomeVFSURI * guri;
 
 	root = gedit_file_browser_store_get_root (store);
 	
@@ -733,27 +1015,18 @@ on_virtual_root_changed_cb (GeditFileBrowserStore * store,
 	                         root,
 	                         NULL);
 	
-	virtual = gedit_file_browser_store_get_virtual_root (store);
+	virtual_root = gedit_file_browser_store_get_virtual_root (store);
 
-	if (!virtual) {
+	if (!virtual_root) {
 		/* Set virtual to same as root then */
 		gconf_client_set_string (client,
 	                                 FILE_BROWSER_BASE_KEY "/on_load/virtual_root",
 	                                 root,
 	                                 NULL);
-	} else {
-		guri = gnome_vfs_uri_new (virtual);
-		gtk_action_set_sensitive (
-			gtk_action_group_get_action (data->action_group, 
-	                                            "OpenTerminal"),
-	                guri != NULL && gedit_file_browser_utils_is_local (virtual));
-		
-		if (guri != NULL)
-			gnome_vfs_uri_unref (guri);
-		
+	} else {		
 		gconf_client_set_string (client,
 	                                 FILE_BROWSER_BASE_KEY "/on_load/virtual_root",
-	                                 virtual,
+	                                 virtual_root,
 	                                 NULL);	
 	}
 
@@ -763,7 +1036,7 @@ on_virtual_root_changed_cb (GeditFileBrowserStore * store,
 
 	g_object_unref (client);
 	g_free (root);
-	g_free (virtual);
+	g_free (virtual_root);
 }
 
 static void 
@@ -774,6 +1047,7 @@ on_tab_added_cb (GeditWindow * window,
 	GConfClient * client;
 	GeditDocument * doc;
 	gboolean open;
+	gchar *uri;
 
 	client = gconf_client_get_default ();
 	
@@ -787,8 +1061,14 @@ on_tab_added_cb (GeditWindow * window,
 	if (open) {
 		doc = gedit_tab_get_document (tab);
 		
-		if (!gedit_document_is_untitled (doc))
-			set_root_from_doc (data, doc);
+		if (!gedit_document_is_untitled (doc)) {
+			uri = gedit_document_get_uri (doc);
+			
+			if (gedit_utils_uri_has_file_scheme (uri)) {
+				prepare_auto_root (data);
+				set_root_from_doc (data, doc);
+			}
+		}
 		
 	}
 	
@@ -801,14 +1081,26 @@ on_tab_added_cb (GeditWindow * window,
 	
 }
 
+static gchar *
+get_filename_from_path (GtkTreeModel *model, GtkTreePath *path)
+{
+	GtkTreeIter iter;
+	gchar *uri;
+
+	gtk_tree_model_get_iter (model, &iter, path);
+	gtk_tree_model_get (model, &iter,
+			    GEDIT_FILE_BROWSER_STORE_COLUMN_URI, &uri,
+			    -1);
+	
+	return gedit_file_browser_utils_uri_basename (uri);
+}
+
 static gboolean
 on_confirm_no_trash_cb (GeditFileBrowserWidget * widget,
                         GeditFileBrowserStore * store,
-                        GtkTreeIter * iter,
+                        GList *paths,
                         GeditWindow * window)
 {
-	gchar *uri;
-	guint flags;
 	gchar *normal;
 	gchar *message;
 	gchar *secondary;
@@ -817,16 +1109,15 @@ on_confirm_no_trash_cb (GeditFileBrowserWidget * widget,
 
 	data = get_plugin_data (window);
 
-	gtk_tree_model_get (GTK_TREE_MODEL (store), iter,
-			    GEDIT_FILE_BROWSER_STORE_COLUMN_URI, &uri,
-			    GEDIT_FILE_BROWSER_STORE_COLUMN_FLAGS, &flags,
-			    -1);
+	message = _("Cannot move file to trash, do you\nwant to delete permanently?");
 
-	message = _("Cannot move file to trash, do you\nwant to delete immediately?");
-
-	normal = gedit_file_browser_utils_uri_basename (uri);
-    	secondary = g_strdup_printf(_("The file \"%s\" cannot be moved to the trash."), normal);
-	g_free (normal);
+	if (paths->next == NULL) {
+		normal = get_filename_from_path (GTK_TREE_MODEL (store), (GtkTreePath *)(paths->data));
+	    	secondary = g_strdup_printf (_("The file \"%s\" cannot be moved to the trash."), normal);
+		g_free (normal);
+	} else {
+		secondary = g_strdup (_("The selected files cannot be moved to the trash."));
+	}
 
 	result = gedit_file_browser_utils_confirmation_dialog (window,
 	                                                       GTK_MESSAGE_QUESTION, 
@@ -842,10 +1133,9 @@ on_confirm_no_trash_cb (GeditFileBrowserWidget * widget,
 static gboolean
 on_confirm_delete_cb (GeditFileBrowserWidget * widget,
                       GeditFileBrowserStore * store,
-                      GtkTreeIter * iter,
+                      GList *paths,
                       GeditWindow * window)
 {
-	gchar *uri;
 	gchar *normal;
 	gchar *message;
 	gchar *secondary;
@@ -854,16 +1144,16 @@ on_confirm_delete_cb (GeditFileBrowserWidget * widget,
 	
 	data = get_plugin_data (window);
 
-	gtk_tree_model_get (GTK_TREE_MODEL (store), iter,
-			    GEDIT_FILE_BROWSER_STORE_COLUMN_URI, &uri,
-			    -1);
-
-	normal = gedit_file_browser_utils_uri_basename  (uri);
-	message =
-	    g_strdup_printf (_
-			     ("Are you sure you want to\npermanently delete \"%s\"?"),
-			     normal);
-	g_free (normal);
+	if (!data->confirm_trash)
+		return TRUE;
+	
+	if (paths->next == NULL) {
+		normal = get_filename_from_path (GTK_TREE_MODEL (store), (GtkTreePath *)(paths->data));
+		message = g_strdup_printf (_("Are you sure you want to\npermanently delete \"%s\"?"), normal);
+		g_free (normal);
+	} else {
+		message = g_strdup (_("Are you sure you want to\npermanently delete the selected files?."));
+	}
 
 	secondary = _("If you delete an item it is permanently lost.");
 
@@ -879,4 +1169,15 @@ on_confirm_delete_cb (GeditFileBrowserWidget * widget,
 	return result;
 }
 
+static void 
+on_end_loading_cb (GeditFileBrowserStore      * store,
+                   GtkTreeIter                * iter,
+                   GeditFileBrowserPluginData * data)
+{
+	/* Disconnect the signal */
+	g_signal_handler_disconnect (store, data->end_loading_handle);
+	data->end_loading_handle = 0;
+	data->auto_root = FALSE;
+}
+
 // ex:ts=8:noet:
